@@ -16,19 +16,20 @@ from .pacman import (
 )
 from .aur import find_aur_packages, AURPackageInfo, strip_aur_repo_name
 from .aur_deps import find_aur_deps, find_repo_deps_of_aur_pkgs
-from .pprint import print_stdout, create_debug_logger
+from .pprint import print_stdout, print_stderr, create_debug_logger, print_error
 from .args import PikaurArgs, parse_args, reconstruct_args
 from .exceptions import DependencyVersionMismatch, DependencyError, SysExit
 from .print_department import print_ignored_package, print_not_found_packages
 from .updates import find_aur_updates, print_upgradeable
 from .replacements import find_replacements
 from .srcinfo import SrcInfo
+from .prompt import ask_to_continue
 
 
 _debug = create_debug_logger('install_info_fetcher')
 
 
-class InstallInfoFetcher(ComparableType):
+class InstallInfoFetcher(ComparableType):  # pylint: disable=too-many-public-methods
 
     repo_packages_install_info: List[RepoInstallInfo]
     new_repo_deps_install_info: List[RepoInstallInfo]
@@ -65,9 +66,13 @@ class InstallInfoFetcher(ComparableType):
         self.pkgbuilds_packagelists = pkgbuilds_packagelists
         self.replacements = find_replacements() if self.args.sysupgrade else {}
 
-        if self.args.sysupgrade:
-            print_upgradeable(ignored_only=True)
         self.get_all_packages_info()
+        if self.args.sysupgrade:
+            # print ignored package updates:
+            print_upgradeable(
+                ignored_only=True,
+                install_infos=self.all_install_info
+            )
 
     def package_is_ignored(self, package_name: str) -> bool:
         ignored_pkg_names = get_ignored_pkgnames_from_patterns(
@@ -111,32 +116,57 @@ class InstallInfoFetcher(ComparableType):
         return list(set(_aur_deps_names))
 
     @property
-    def aur_install_info(self) -> Sequence[List[AURInstallInfo]]:
+    def aur_install_info_containers(self) -> Sequence[List[AURInstallInfo]]:
         return (
             self.aur_updates_install_info,
             self.aur_deps_install_info,
         )
 
     @property
-    def repo_install_info(self) -> Sequence[List[RepoInstallInfo]]:
+    def repo_install_info_containers(self) -> Sequence[List[RepoInstallInfo]]:
         return (
             self.repo_packages_install_info,
             self.new_repo_deps_install_info,
+            self.repo_replacements_install_info,
             self.thirdparty_repo_packages_install_info,
+            self.new_thirdparty_repo_deps_install_info,
+            self.thirdparty_repo_replacements_install_info,
         )
 
     @property
-    def all_install_info(self) -> Sequence[Union[List[RepoInstallInfo], List[AURInstallInfo]]]:
+    def all_install_info_containers(
+            self
+    ) -> Sequence[Union[List[RepoInstallInfo], List[AURInstallInfo]]]:
         return (
-            *self.repo_install_info,
-            *self.aur_install_info,
+            *self.repo_install_info_containers,
+            *self.aur_install_info_containers,
         )
 
+    @property
+    def repo_install_info(self) -> Sequence[RepoInstallInfo]:
+        return list(
+            info
+            for infos in self.repo_install_info_containers
+            for info in infos
+        )
+
+    @property
+    def aur_install_info(self) -> Sequence[AURInstallInfo]:
+        return list(
+            info
+            for infos in self.aur_install_info_containers
+            for info in infos
+        )
+
+    @property
+    def all_install_info(self) -> Sequence[Union[RepoInstallInfo, AURInstallInfo]]:
+        return list(self.repo_install_info) + list(self.aur_install_info)
+
     def discard_package(
             self, canceled_pkg_name: str, already_discarded: List[str] = None
     ) -> List[str]:
         _debug(f"discarding {canceled_pkg_name=}")
-        for container in self.all_install_info:
+        for container in self.all_install_info_containers:
             for info in container[:]:
                 if info.name == canceled_pkg_name:
                     container.remove(info)  # type: ignore[arg-type]
@@ -191,7 +221,7 @@ class InstallInfoFetcher(ComparableType):
 
         self.mark_dependent()
 
-    def _get_repo_pkgs_info(
+    def _get_repo_pkgs_info(  # pylint: disable=too-many-statements,too-many-locals,too-many-branches
             self, pkg_lines: List[str], extra_args: Optional[List[str]] = None
     ) -> List[RepoInstallInfo]:
         if not pkg_lines:
@@ -243,6 +273,7 @@ class InstallInfoFetcher(ComparableType):
             return install_infos
 
         try:
+            _debug(f"Checking if '{pkg_lines=}' is installable:")
             composed_result = PackageDB.get_print_format_output(
                 pacman_args + [
                     pkg
@@ -251,25 +282,54 @@ class InstallInfoFetcher(ComparableType):
                 ]
             )
         except DependencyError:
+            _debug("Check failed - gonna check it separately:")
             composed_result = []
         if composed_result:
+            _debug("Check passed - gonna get install infos:")
             return _get_pkg_install_infos(composed_result)
 
         all_results = {}
         with ThreadPool() as pool:
-            all_requests = {}
+            pkg_exists_requests = {}
             for pkg_name in pkg_lines[:]:
-                all_requests[pkg_name] = pool.apply_async(
+                _debug(f"Checking if '{pkg_name}' exists in the repo:")
+                pkg_exists_requests[pkg_name] = pool.apply_async(
+                    PackageDB.get_print_format_output,
+                    (pacman_args + pkg_name.split(','), False, True)
+                )
+            pool.close()
+            pool.join()
+            for pkg_name, request in pkg_exists_requests.items():
+                try:
+                    all_results[pkg_name] = request.get()
+                    _debug(f"  '{pkg_name}' is found in the repo.")
+                except DependencyError:
+                    all_results[pkg_name] = []
+                    _debug(f"  '{pkg_name}' is NOT found in the repo.")
+
+        with ThreadPool() as pool:
+            pkg_installable_requests = {}
+            for pkg_name, results in all_results.items():
+                if not results:
+                    continue
+                _debug(f"Checking if '{pkg_name}' is installable:")
+                pkg_installable_requests[pkg_name] = pool.apply_async(
                     PackageDB.get_print_format_output,
                     (pacman_args + pkg_name.split(','), )
                 )
             pool.close()
             pool.join()
-            for pkg_name, request in all_requests.items():
+            for pkg_name, request in pkg_installable_requests.items():
                 try:
-                    all_results[pkg_name] = request.get()
-                except DependencyError:
-                    all_results[pkg_name] = []
+                    request.get()
+                    _debug(f"  '{pkg_name}' is installable.")
+                except DependencyError as exc:
+                    print_error(f"'{pkg_name}' is NOT installable.")
+                    print_stderr(str(exc))
+                    if not ask_to_continue:
+                        raise SysExit(125) from exc
+
+        _debug("Check partially passed - gonna get install infos:")
         pkg_install_infos: List[RepoInstallInfo] = []
         for pkg_name, results in all_results.items():
             if not results:
@@ -312,17 +372,7 @@ class InstallInfoFetcher(ComparableType):
                 self._get_repo_pkgs_info(pkg_lines=self.install_package_names) +
                 self.get_upgradeable_repo_pkgs_info()
         ):
-            if pkg_update.name in [
-                    install_info.name for install_info in
-                    (
-                        self.repo_packages_install_info +
-                        self.thirdparty_repo_packages_install_info +
-                        self.new_repo_deps_install_info +
-                        self.new_thirdparty_repo_deps_install_info +
-                        self.repo_replacements_install_info +
-                        self.thirdparty_repo_replacements_install_info
-                    )
-            ]:
+            if pkg_update.name in self.repo_install_info:
                 continue
 
             pkg_name = pkg_update.name
@@ -527,16 +577,6 @@ class InstallInfoFetcher(ComparableType):
 
         all_provided_pkgs = PackageDB.get_repo_provided_dict()
         all_local_pkgnames = PackageDB.get_local_pkgnames()
-        all_install_infos: Sequence[InstallInfo] = (
-            self.repo_packages_install_info +
-            self.thirdparty_repo_packages_install_info +
-            self.new_repo_deps_install_info +
-            self.new_thirdparty_repo_deps_install_info +
-            self.aur_updates_install_info +
-            self.aur_deps_install_info +
-            self.repo_replacements_install_info +
-            self.thirdparty_repo_replacements_install_info  # type: ignore[operator]
-        )
         all_deps_install_infos: Sequence[InstallInfo] = (
             self.new_repo_deps_install_info +
             self.new_thirdparty_repo_deps_install_info +
@@ -548,12 +588,12 @@ class InstallInfoFetcher(ComparableType):
             ) if isinstance(ii.package, AURPackageInfo) else (
                 ii.package.depends
             )
-            for ii in all_install_infos
+            for ii in self.all_install_info
         ], [])
         explicit_aur_pkg_names = [ii.name for ii in self.aur_updates_install_info]
 
         # iterate each package metadata
-        for pkg_install_info in all_install_infos:
+        for pkg_install_info in self.all_install_info:
 
             # process providers
             provides = pkg_install_info.package.provides
@@ -620,12 +660,12 @@ class InstallInfoFetcher(ComparableType):
 
     def get_total_download_size(self) -> float:
         total_download_size = 0.0
-        for install_info in chain.from_iterable(self.repo_install_info):
+        for install_info in chain.from_iterable(self.repo_install_info_containers):
             total_download_size += install_info.package.size / 1024 ** 2
         return total_download_size
 
     def get_total_installed_size(self) -> float:
         total_installed_size = 0.0
-        for install_info in chain.from_iterable(self.repo_install_info):
+        for install_info in chain.from_iterable(self.repo_install_info_containers):
             total_installed_size += install_info.package.isize / 1024 ** 2
         return total_installed_size
